local L = LibStub("AceLocale-3.0"):GetLocale("AVRE", false)
local AVRE = AVRE
if not AVRE or not AVR then return end
local mod = AVRE:NewModule("Coliseum")
local zone = LibStub("AceLocale-3.0"):GetLocale("AVRE Zones")["Trial of the Crusader"]
local chat = LibStub("AceLocale-3.0"):GetLocale("AVRE Coliseum")


function mod:OnInitialize()
	AVRE.modules.encounters.coliseum.loaded = true
end


function mod:OnEnable()
	DEFAULT_CHAT_FRAME:AddMessage(format(L["|cff4d4dffAVR Encounters (|r|cffffff20%s|r|cff4d4dff) loaded|r"], zone))
	
	AVRE:RegisterEncounters({

-- Beast of Northrend --------------------------------------------------------------------------------------------------------------

		beast = {
			enable = {
				expansion = "wotlk",
				instance = "coliseum",
				fight = "beast",
			},
			name = L["Beast of Northrend"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 34796 },
			},
			defeat = {
				type = "death",
				npcs = { 34797 }, -- This won't actually work, but we should detect a "wipe" anyway so it doesn't matter
			},
			
			
			onStart = function(self)
				self.snowbolled = {}
				self.circles = {}
				self.markers = {}
			end,
			
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_SUMMON = true,
				UNIT_DIED = true,
				SPELL_AURA_APPLIED = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				if eventType == "SPELL_SUMMON" then
				
					-- Snowboll summoned
					if spellId == 66406 then
						if self.o("snowbolled") then
							self.snowbolled[dstGuid] = srcGuid
							local a = self.a("snowbolled")
							local m = AVR:AddCircleWarning(AVRE.scene, srcName, true, 2, nil, a.r, a.g, a.b, a.a, nil, true)
							if m then
								m:SetTimer(8)
								self.circles[dstGuid] = m
								
								m = AVRMarkerMesh:New()
								m:SetFollowUnit(srcName)
								m:SetLine(true)
								m:SetColor(1.0, 1.0, 1.0, 0.8)
								AVRE.scene:AddMesh(m)
								self.markers[dstGuid] = m
							end
						end
					end
					
				elseif eventType == "UNIT_DIED" then
					
					-- Snowboll killed
					if self.snowbolled[dstGuid] then
						if self.circles[dstGuid] then
							self.circles[dstGuid]:Remove()
							self.circles[dstGuid] = nil
						end
						if self.markers[dstGuid] then
							self.markers[dstGuid]:Remove()
							self.markers[dstGuid] = nil
						end
						self.snowbolled[dstGuid] = nil
					end
					
					-- Snowbolled player killed
					for k, v in pairs(self.snowbolled) do
						if v == dstGuid then
							if self.circles[k] then
								self.circles[k]:Remove()
								self.circles[k] = nil
							end
							if self.markers[k] then
								self.markers[k]:Remove()
								self.markers[k] = nil
							end
							self.snowbolled[k] = nil
						end
					end
					
				elseif eventType == "SPELL_AURA_APPLIED" then
				
					--Burning Bile
					
					if (spellId == 67621 or spellId == 67623 or spellId == 67622 or spellId == 66870 or spellId == 66869) and self.o("bile") then
						local a = self.a("bile")
						AVR:AddCircleWarning(AVRE.scene, dstName, true, 5, 30, a.r, a.g, a.b, a.a, nil, true)
					
					--Crash
					
					elseif spellId == 52311 and self.o("crash") then
					local a = self.a("crash")
					AVRE:GetBossTarget(34797, function(target)		
						AVR:AddCircleWarning(AVRE.scene, target, false, 15, 7.5, a.r, a.g, a.b, a.a)
					end)
					end
					
				end
			end,
		},


})
	
end
