local L = LibStub("AceLocale-3.0"):GetLocale("AVRE", false)
local AVRE = AVRE
local mod = AVRE:NewModule("Options", "AceConsole-3.0")
local zones = LibStub("AceLocale-3.0"):GetLocale("AVRE Zones")



local newOrder
do
	local current = 0
	function newOrder()
		current = current + 1
		return current
	end
end

local function getRaidUnits(current)
	local t = { [""] = "" }
	
	if current and current ~= "" then t[current] = AVRE:Colorize(current, GRAY_FONT_COLOR) end
	
	if UnitInRaid("player") then
		local numRaid = GetNumRaidMembers()
		for i = 1, numRaid do
			local playerName = GetRaidRosterInfo(i)
			if playerName then
				t[playerName] = AVRE:Colorize(playerName, (RAID_CLASS_COLORS[({UnitClass(playerName)})[2]]))
			end
		end
	end
	
	return t
end

local newExpansion
local newInstance
local newFight
local a
local b
local c

local function newExpansion(n, s)
	a = s
	return n
end

local function newInstance(n, s)
	b = s
	return n
end

local function newFight(n, s)
	c = s
	return n
end

local function checkbox_spell(a, b, c, spellId, message, option)
	return {
		type = "toggle",
		name = GetSpellInfo(spellId),
		desc = format(message, GetSpellInfo(spellId)),
		get = function() return AVRE.db.profile.enc[a][b][c][option] end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c][option] = val end,
		order = newOrder(),
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled end,
	}
end

local function checkbox(a, b, c, name, option, desc)
	return {
		type = "toggle",
		name = name,
		desc = desc,
		get = function() return AVRE.db.profile.enc[a][b][c][option] end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c][option] = val end,
		order = newOrder(),
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled and option ~= "enabled" end,
	}
end

local function splitter()
	return {
		type = "header",
		name = "",
		order = newOrder(),
	}
end

local function spacer()
	return {
		type = "description",
		name = " ",
		order = newOrder(),
	}
end

local function header(name)
	return {
		type = "header",
		order = newOrder(),
		name = name,
	}
end

local function header_spell(spellId)
	return {
		type = "header",
		order = newOrder(),
		name = GetSpellInfo(spellId),
	}
end

local function desc_spell(spellId, msg)
	return {
		type = "description",
		order = newOrder(),
		name = msg:format(GetSpellInfo(spellId)),
	}
end

local function desc(msg)
	return {
		type = "description",
		order = newOrder(),
		name = msg,
	}
end

local function range1()
	return header(L["Range Check"])
end

local function range2(a, b, c, extraDesc)
	return {
		type = "toggle",
		name = L["Enable range check"],
		desc = L["Turns on an AVR range check when the fight starts."].." "..(extraDesc or ""),
		get = function() return AVRE.db.profile.enc[a][b][c].range.enabled end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c].range.enabled = val end,
		order = newOrder(),
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled end,
	}
end

local function range3(a, b, c)
	return {
		order = newOrder(),
		type = "range",
		name = L["Range"],
		desc = function()
			local def = AVRE.defaults.profile.enc[a][b][c].range.range
			return format(L["The distance in yards for the AVR range check."].." "..L["(Default is %d yards.)"], def)
		end,
		min = 1,
		max = 30,
		step = 1,
		get = function() return AVRE.db.profile.enc[a][b][c].range.range end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c].range.range = val end,
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled end,
	}
end

local function range_adv(a, b, c, option)
	return {
		order = newOrder(),
		type = "range",
		name = L["Range"],
		desc = function()
			local def = AVRE.defaults.profile.enc[a][b][c].advanced[option].range
			return format(L["The distance in yards for the AVR range check."].." "..L["(Default is %d yards.)"], def)
		end,
		min = 1,
		max = 30,
		step = 1,
		get = function() return AVRE.db.profile.enc[a][b][c].advanced[option].range end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c].advanced[option].range = val end,
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled end,
	}
end

local function character_picker(a, b, c, name, option, desc)
	return {
		order = newOrder(),
		type = "select",
		name = name,
		desc = desc,
		values = function() return getRaidUnits(AVRE.db.profile.enc[a][b][c][option]) end,
		get = function() return AVRE.db.profile.enc[a][b][c][option] end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c][option] = val end,
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled end,
	}
end

local function color(a, b, c, option)
	return {
		order = newOrder(),
		type = "color",
		name = L["Color and transparency"],
		hasAlpha = true,
		set = function(self, red, green, blue, alpha)
			AVRE.db.profile.enc[a][b][c].advanced[option].r = red
			AVRE.db.profile.enc[a][b][c].advanced[option].g = green
			AVRE.db.profile.enc[a][b][c].advanced[option].b = blue
			AVRE.db.profile.enc[a][b][c].advanced[option].a = alpha
		end,
		get = function()
			return AVRE.db.profile.enc[a][b][c].advanced[option].r, AVRE.db.profile.enc[a][b][c].advanced[option].g, AVRE.db.profile.enc[a][b][c].advanced[option].b, AVRE.db.profile.enc[a][b][c].advanced[option].a
		end,
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled end,
	}
end

local function radius(a, b, c, option)
	return {
		order = newOrder(),
		type = "range",
		name = L["Radius"],
		desc = function()
			local def = AVRE.defaults.profile.enc[a][b][c].advanced[option].rad
			return format(L["Determines the radius of the circle in yards."].." "..L["(Default is %.1f yards.)"], def)
		end,
		min = 0,
		max = 30,
		bigStep = 1,
		get = function() return AVRE.db.profile.enc[a][b][c].advanced[option].rad end,
		set = function(self, val) AVRE.db.profile.enc[a][b][c].advanced[option].rad = val end,
		disabled = function() return not AVRE.db.profile.enc[a][b][c].enabled end,
	}
end

local function broadcast(a, b, c, option, description)
	return {
		type = "execute",
		name = L["Broadcast to raid"],
		desc = description,
		func = function()
			AVRE:Sync({ c = strupper("ENC_"..c), v = AVRE.db.profile.enc[a][b][c][option] })
		end,
		order = newOrder(),
		disabled = function() return AVRE.db.profile.enc[a][b][c][option] == "" end,
	}
end



--[[
      OPTIONS ------------------------------------------------------------------------------------------------------------------
  ]]


AVRE.options = {
	name = function()
		local debugMsg = "    "
		--[===[@debug@
		debugMsg = debugMsg..L["(running in debug mode)"]
		--@end-debug@]===]
		return "AVR Encounters         r|cff20ff20"..AVRE.MINOR_VERSION.."|r"..debugMsg
	end,
	handler = VampArrow,
	type = "group",
	childGroups = "tab",
	args = {
	
		encounters = {
			type = "group",
			name = L["Encounters"],
			order = newOrder(),
			args = {
	
--------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------
-- WRATH OF THE LICH KING ------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------

				wotlk = {
					name = newExpansion(L["Wrath of the Lich King"], "wotlk"),
					type = "group",
					order = newOrder(),
					args = {
					
						enabled = {
							type = "toggle",
							name = L["Enable Expansion"],
							get = function() return AVRE.db.profile.enc.wotlk.enabled end,
							set = function(self, val) AVRE.db.profile.enc.wotlk.enabled = val end,
							order = newOrder(),
						},


--------------------------------------------------------------------------------------------------------------------
-- Coliseum By Esa----------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------

						coliseum = {
							name = newInstance(zones["Trial of the Crusader"], "coliseum"),
							type = "group",
							order = newOrder(),
							args = {
								
								enabled = {
									type = "toggle",
									name = L["Enable Raid Instance"],
									get = function() return AVRE.db.profile.enc.wotlk.coliseum.enabled end,
									set = function(self, val) AVRE.db.profile.enc.wotlk.coliseum.enabled = val end,
									order = newOrder(),
								},
-- Beast of Northrend -----------------------------------------------------------------------------------------------------------

								beast = {
									name = newFight(L["Beast of Northrend"], "beast"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										snowbolled1 = header_spell(66406),
										snowbolled2 = desc_spell(66406, L["Places a circle under players with %s."]),
										snowbolled3 = checkbox(a,b,c, L["Enabled"], "snowbolled"),
										snowbolled4 = color(a,b,c, "snowbolled"),
										
										spacer2 = spacer(),
										bile1 = header_spell(67621),
										bile2 = desc_spell(67621, L["Places a circle under players with %s."]),
										bile3 = checkbox(a,b,c, L["Enabled"], "bile"),
										bile4 = color(a,b,c, "bile"),		
										
										spacer3 = spacer(),
										crash1 = header_spell(66683),
										crash2 = desc_spell(66683, L["Places a circle under players with %s."]),
										crash3 = checkbox(a,b,c, L["Enabled"], "crash"),
										crash4 = color(a,b,c, "crash"),									
									},
								},
							},
						},								
--------------------------------------------------------------------------------------------------------------------
-- ULDUAR ----------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------

						ulduar = {
							name = newInstance(zones["Ulduar"], "ulduar"),
							type = "group",
							order = newOrder(),
							args = {
								
								enabled = {
									type = "toggle",
									name = L["Enable Raid Instance"],
									get = function() return AVRE.db.profile.enc.wotlk.ulduar.enabled end,
									set = function(self, val) AVRE.db.profile.enc.wotlk.ulduar.enabled = val end,
									order = newOrder(),
								},
-- XT-002 -----------------------------------------------------------------------------------------------------------

								xt002 = {
									name = newFight(L["XT-002 Deconstructor"], "xt002"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										Light1 = header_spell(65121),
										Light2 = desc_spell(65121, L["Places a circle under players with %s."]),
										Light3 = checkbox(a,b,c, L["Enabled"], "searingLight"),
										Light4 = color(a,b,c, "searingLight"),
										
										spacer2 = spacer(),
										Bomb1 = header_spell(64234),
										Bomb2 = desc_spell(64234, L["Places a circle under players with %s."]),
										Bomb3 = checkbox(a,b,c, L["Enabled"], "gravityBomb"),
										Bomb4 = color(a,b,c, "gravityBomb"),										
									},
								},
-- Kologarn -----------------------------------------------------------------------------------------------------------

								kologarn = {
									name = newFight(L["Kologarn"], "kologarn"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										eye1 = header_spell(63346),
										eye2 = desc_spell(63346, L["Places a circle under players with %s."]),
										eye3 = checkbox(a,b,c, L["Enabled"], "eye"),
										eye4 = color(a,b,c, "eye"),
									},
								},
								
-- HODIR -----------------------------------------------------------------------------------------------------------

								hodir = {
									name = newFight(L["Hodir"], "hodir"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										storm1 = header_spell(65123),
										storm2 = desc_spell(65123, L["Places a circle under players with %s."]),
										storm3 = checkbox(a,b,c, L["Enabled"], "storm"),
										storm4 = color(a,b,c, "storm"),
									},
								},

-- FREYA -----------------------------------------------------------------------------------------------------------

								freya = {
									name = newFight(L["Freya"], "freya"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										root1 = header_spell(62861),
										root2 = desc_spell(62861, L["Places a circle under players with %s."]),
										root3 = checkbox(a,b,c, L["Enabled"], "root"),
										root4 = color(a,b,c, "root"),

										spacer2 = spacer(),
										fury1 = header_spell(63571),
										fury2 = desc_spell(63571, L["Places a circle under players with %s."]),
										fury3 = checkbox(a,b,c, L["Enabled"], "fury"),
										fury4 = color(a,b,c, "fury"),
									},
								},

-- MIMIRON ---------------------------------------------------------------------------------------------------------

								mimiron = {
									name = newFight(L["Mimiron"], "mimiron"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										napalm1 = header_spell(63666),
										napalm2 = desc_spell(63666, L["Places a circle under players with %s."]),
										napalm3 = checkbox(a,b,c, L["Enabled"], "napalm"),
										napalm4 = color(a,b,c, "napalm"),
									},
								},

-- GENERAL VEZAX ---------------------------------------------------------------------------------------------------

								generalvezax = {
									name = newFight(L["General Vezax"], "generalvezax"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										crash1 = header_spell(62660),
										crash2 = desc_spell(62660, L["Places a circle under players with %s."]),
										crash3 = checkbox(a,b,c, L["Enabled"], "crash"),
										crash4 = color(a,b,c, "crash"),

										spacer2 = spacer(),
										mark1 = header_spell(63276),
										mark2 = desc_spell(63276, L["Places a circle under players with %s."]),
										mark3 = checkbox(a,b,c, L["Enabled"], "mark"),
										mark4 = color(a,b,c, "mark"),
									},
								},
								
-- YOGG-SARON ------------------------------------------------------------------------------------------------------

								yoggsaron = {
									name = newFight(L["Yogg-Saron"], "yoggsaron"),
									type = "group",
									order = newOrder(),
									args = {
										enabled = checkbox(a,b,c, L["Enable Encounter"], "enabled"),
										
										spacer1 = spacer(),
										malady1 = header_spell(63830),
										malady2 = desc_spell(63830, L["Places a circle under players with %s."]),
										malady3 = checkbox(a,b,c, L["Enabled"], "malady"),
										malady4 = color(a,b,c, "malady"),
										
										spacer2 = spacer(),
										squeeze1 = header_spell(64125),
										squeeze2 = desc_spell(64125, L["Places a circle under players with %s."]),
										squeeze3 = checkbox(a,b,c, L["Enabled"], "squeeze"),
										squeeze4 = color(a,b,c, "squeeze"),
										
										spacer3 = spacer(),
										crusher1 = header_spell(64147),
										crusher2 = desc_spell(64147, L["Places a circle under players with %s."]),
										crusher3 = checkbox(a,b,c, L["Enabled"], "crusher"),
										crusher4 = color(a,b,c, "crusher"),
									},
								},



							},
						},

		
					},
				},
				
			},
		},





























		options = {
			type = "group",
			name = L["General Options"],
			order = newOrder(),
			args = {
			
				enabled = {
					type = "toggle",
					name = L["Enable AVR Encounters"],
					get = function() return AVRE.db.profile.enabled end,
					set = function(self, val)
						AVRE.db.profile.enabled = val
						if val then AVRE:Enable() else AVRE:Disable() end
					end,
					order = newOrder(),
				},
				
				range = {
					type = "group",
					name = L["Range Checks"],
					order = newOrder(),
					args = {
					
						header1 = header(L["General Range Checks"]),
						desc1 = desc(L["Certain fights offer the option to automatically turn on the AVR range check for the entire duration of the fight. The AVR range check is a special AVR scene that assists you in locating in-range players using the 3D game world instead of a list of names. It draws a circular marker under players in range, and can optionally draw a line to those players as well. Finally, a circle outline can optionally be drawn around you, assisting you in knowing when you're approaching range of other players.\n\nYou can see how AVR range checks work by creating a range check scene yourself using AVR. Play with the options to see what you like, then configure them here in AVR Encounters. |cffffff20Be aware that AVR range checks are very CPU intensive.|r"]),

						classColor = {
							type = "toggle",
							name = L["Color by class"],
							desc = L["When enabled, lines and markers to in-range players will be colored by that player's class."],
							get = function() return AVRE.db.profile.range.classColor end,
							set = function(self, val) AVRE.db.profile.range.classColor = val end,
							order = newOrder(),
						},
	
						line = {
							type = "toggle",
							name = L["Show lines to players"],
							desc = L["When enabled, a line will be drawn from you to any player within range of you."],
							get = function() return AVRE.db.profile.range.line end,
							set = function(self, val) AVRE.db.profile.range.line = val end,
							order = newOrder(),
						},
	
						circle = {
							type = "toggle",
							name = L["Show range circle outline"],
							desc = L["When enabled, a circle outline will be drawn around you, indicating the current range."],
							get = function() return AVRE.db.profile.range.circle end,
							set = function(self, val) AVRE.db.profile.range.circle = val end,
							order = newOrder(),
						},
	
						radius = {
							order = newOrder(),
							type = "range",
							name = L["Marker radius"],
							desc = format(L["Determines the radius of the markers drawn under players within range of you."].." "..L["(Default is %.1f yards.)"], 0.5),
							min = 0,
							max = 5,
							bigStep = 0.1,
							get = function() return AVRE.db.profile.range.radius end,
							set = function(self, val) AVRE.db.profile.range.radius = val end,
						},
						
						spacer2 = spacer(),
						header2 = header(L["Mechanic-Specific Range Checks"]),
						desc2 = desc(L["The following options control AVR range checks that appear during specific fight mechanics. These options do not affect general range checks that are used throughout an entire fight."]),
	
						circleSelfEnabled = {
							type = "toggle",
							name = L["Show circle around self"],
							desc = L["When enabled, a filled circle will be drawn around you, indicating the current range."],
							get = function() return AVRE.db.profile.range.circleSelf.enabled end,
							set = function(self, val) AVRE.db.profile.range.circleSelf.enabled = val end,
							order = newOrder(),
						},
						
						circleSelfColor = {
							order = newOrder(),
							type = "color",
							name = L["Self circle color"],
							desc = L["Determines the color and transparency of the filled circle drawn around you during mechanic-specific range checks."],
							hasAlpha = true,
							set = function(self, red, green, blue, alpha)
								AVRE.db.profile.range.circleSelf.r = red
								AVRE.db.profile.range.circleSelf.g = green
								AVRE.db.profile.range.circleSelf.b = blue
								AVRE.db.profile.range.circleSelf.a = alpha
							end,
							get = function()
								return AVRE.db.profile.range.circleSelf.r, AVRE.db.profile.range.circleSelf.g, AVRE.db.profile.range.circleSelf.b, AVRE.db.profile.range.circleSelf.a
							end,
							disabled = function() return not AVRE.db.profile.range.circleSelf.enabled end,
						},
	
					},
				},
				
			},
		},
					
	}
}



--[[
      INITIALIZATION -----------------------------------------------------------------------------------------------------------
  ]]

function mod:OnInitialize()
	-- Remove options for disabled modules
	--@non-debug@
	for k, v in pairs(AVRE.modules.encounters) do
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(v.addon)
		if not enabled then
			AVRE.options.args.encounters.args[v.expansion].args[k] = nil
			local count = 0
			for kk, vv in pairs(AVRE.options.args.encounters.args[v.expansion].args) do
				count = count + 1
			end
			if count <= 1 then
				AVRE.options.args.encounters.args[v.expansion] = nil
				count = 0
				for kkk, vvv in pairs(AVRE.options.args.encounters.args) do
					count = count + 1
				end
				if count <= 0 then
					AVRE.options.args.encounters.args.allDisabled = desc(L["All AVRE encounter modules have been disabled. Enable one or more of them from the add-on selection screen in order to configure them."])
				end
			end
		end
	end
	--@end-non-debug@
	
	-- Register options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AVRE", AVRE.options)
	AVRE.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AVRE", "AVR Encounters")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("AVRE-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(AVRE.db))
	AVRE.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("AVRE-Profiles", "Profiles", "AVR Encounters")
	AVRE.modules.options.loaded = true
end


function mod:OnEnable()
--[===[@debug@
	self:Print(L["Loaded"])
--@end-debug@]===]
end
