local L = LibStub("AceLocale-3.0"):GetLocale("AVRE", false)
local AVRE = AVRE
if not AVRE or not AVR then return end
local mod = AVRE:NewModule("Ulduar")
local zone = LibStub("AceLocale-3.0"):GetLocale("AVRE Zones")["Ulduar"]
local chat = LibStub("AceLocale-3.0"):GetLocale("AVRE Ulduar")


function mod:OnInitialize()
	AVRE.modules.encounters.ulduar.loaded = true
end


function mod:OnEnable()
	DEFAULT_CHAT_FRAME:AddMessage(format(L["|cff4d4dffAVR Encounters (|r|cffffff20%s|r|cff4d4dff) loaded|r"], zone))
	
	AVRE:RegisterEncounters({

-- XT-002---------------------------------------------------------------------------------------------------------

		xt002 = {
			enable = {
				expansion = "wotlk",
				instance = "ulduar",
				fight = "xt002",
			},
			name = L["XT-002 Deconstructor"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 33293 },
			},
			defeat = {
				type = "death",
				npcs = { 33293 },
			},
			
			onStart = function(self) end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_AURA_APPLIED = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				-- Searing Light
				if (spellId == 63018 or spellId == 65121) and self.o("searingLight") then
					local a = self.a("searingLight")
					AVR:AddCircleWarning(AVRE.scene, dstName, true, 10, 9, a.r, a.g, a.b, a.a)
				-- Gravity Bomb
				elseif (spellId == 63024 or spellId == 64234 or spellId == 64233 or spellId == 64225) and self.o("gravityBomb") then
					local a = self.a("gravityBomb")
					AVR:AddCircleWarning(AVRE.scene, dstName, true, 20, 9, a.r, a.g, a.b, a.a)
				end
			end,
		},
	
-- Kologarn ------------------------------------------------------------------------------------------------------------

		kologarn = {
			enable = {
				expansion = "wotlk",
				instance = "ulduar",
				fight = "kologarn",
			},
			name = L["Kologarn"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 32930 },
			},
			defeat = {
				type = "death",
				npcs = { 32930 }, -- This won't actually work, but we should detect a "wipe" anyway so it doesn't matter
			},
			
			onStart = function(self) end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_CAST_SUCCESS = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				-- Eye Beam
				if (spellId == 63346 or spellId == 63976) and self.o("eye") then
					local a = self.a("eye")
							AVRE:GetBossTarget(32930, function(target)		
								AVR:AddCircleWarning(AVRE.scene, target, true, 5, 10, a.r, a.g, a.b, a.a)
							end)
				end
			end,
		},		
-- HODIR --------------------------------------------------------------------------------------------------------------

		hodir = {
			enable = {
				expansion = "wotlk",
				instance = "ulduar",
				fight = "hodir",
			},
			name = L["Hodir"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 32845 },
			},
			defeat = {
				type = "death",
				npcs = { 32845 }, -- This won't actually work, but we should detect a "wipe" anyway so it doesn't matter
			},
			
			onStart = function(self)
				self.storm = {}
			end,
			
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_REMOVED = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				if eventType == "SPELL_AURA_APPLIED" then
					-- Storm Cloud applied
					if (spellId == 65123 or spellId == 65133) and self.o("storm") then
						if self.storm[dstName] then
							self.storm[dstName]:Remove()
							self.storm[dstName] = nil
						end
						local a = self.a("storm")
						self.storm[dstName] = AVR:AddCircleWarning(AVRE.scene, dstName, true, 3, 30, a.r, a.g, a.b, a.a, nil, true)
					end
					
				elseif eventType == "SPELL_AURA_REMOVED" then
					-- Storm Cloud removed
					if spellId == 65123 or spellId == 65133 then
						if self.storm[dstName] then
							self.storm[dstName]:Remove()
							self.storm[dstName] = nil
						end
					end

				end
			end,
		},


-- FREYA --------------------------------------------------------------------------------------------------------------

		freya = {
			enable = {
				expansion = "wotlk",
				instance = "ulduar",
				fight = "freya",
			},
			name = L["Freya"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 32906 },
			},
			defeat = {
				type = "death",
				npcs = { 32906 }, -- This won't actually work, but we should detect a "wipe" anyway so it doesn't matter
			},
			
			onStart = function(self)
				self.root = {}
			end,
			
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_AURA_APPLIED = true,
				SPELL_AURA_REMOVED = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				if eventType == "SPELL_AURA_APPLIED" then
					-- Iron Roots applied
					if (spellId == 62861 or spellId == 62438) and self.o("root") then
						if self.root[dstName] then
							self.root[dstName]:Remove()
							self.root[dstName] = nil
						end
						local a = self.a("root")
						self.root[dstName] = AVR:AddCircleWarning(AVRE.scene, dstName, true, 3, nil, a.r, a.g, a.b, a.a, nil, false)
					
					-- Nature's Fury
					elseif (spellId == 63571 or spellId == 62589) and self.o("fury") then
						local a = self.a("fury")
						AVR:AddCircleWarning(AVRE.scene, dstName, true, 8, 10, a.r, a.g, a.b, a.a, nil, true)
					
					end
					
				elseif eventType == "SPELL_AURA_REMOVED" then
					-- Iron Roots removed
					if spellId == 62861 or spellId == 62438 then
						if self.root[dstName] then
							self.root[dstName]:Remove()
							self.root[dstName] = nil
						end
					end

				end
			end,
		},


-- MIMIRON ------------------------------------------------------------------------------------------------------------

		mimiron = {
			enable = {
				expansion = "wotlk",
				instance = "ulduar",
				fight = "mimiron",
			},
			name = L["Mimiron"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 33432 },
			},
			defeat = {
				type = "death",
				npcs = { 33432 }, -- This won't actually work, but we should detect a "wipe" anyway so it doesn't matter
			},
			
			onStart = function(self) end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_AURA_APPLIED = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				-- Napalm Shell
				if (spellId == 63666 or spellId == 65026) and self.o("napalm") then
					local a = self.a("napalm")
					AVR:AddCircleWarning(AVRE.scene, dstName, false, 5, 8, a.r, a.g, a.b, a.a, nil, true)
				end
			end,
		},


	-- GENERAL VEZAX ------------------------------------------------------------------------------------------------------

		generalvezax = {
			enable = {
				expansion = "wotlk",
				instance = "ulduar",
				fight = "generalvezax",
			},
			name = L["General Vezax"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 33271 },
			},
			defeat = {
				type = "death",
				npcs = { 33271 },
			},
			
			onStart = function(self) end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_CAST_SUCCESS = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				
				-- Shadow Crash
				if (spellId == 62659 or spellId == 63277 or spellId == 62660 or spellId == 60835) and self.o("crash") then
					local a = self.a("crash")
										
						AVRE:GetBossTargetChange(33271, function(target)
						AVR:AddCircleWarning(AVRE.scene, target, false, 10, 7, a.r, a.g, a.b, a.a)
					end)
					
				-- Mark of the Faceless
				elseif spellId == 63276 and self.o("mark") then
					local a = self.a("mark")
					AVR:AddCircleWarning(AVRE.scene, dstName, true, 15, 10, a.r, a.g, a.b, a.a, nil, true)
				end

			end,
		},


-- YOGG-SARON ---------------------------------------------------------------------------------------------------------

		yoggsaron = {
			enable = {
				expansion = "wotlk",
				instance = "ulduar",
				fight = "yoggsaron",
			},
			name = L["Yogg-Saron"],
			zone = zone,
			pull = {
				type = "combat",
				npcs = { 33134 },
			},
			defeat = {
				type = "death",
				npcs = { 33288 },
			},
			
			onStart = function(self)
				self.squeeze = {}
				self.circles = {}
				self.markers = {}
				function(srcName)
			 end,
			onEnd = function(self) end,
			
			combatEvents = {
				SPELL_SUMMON = true,
				UNIT_DIED = true,
				SPELL_AURA_APPLIED = true,
				SPELL_CAST_SUCCESS = true,
			},
			
			onCombat = function(self, event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, spellId, spellName, spellSchool)
				if eventType == "SPELL_SUMMON" then
				
					-- Squeeze summoned
					if spellId == 64125 or spellId == 64126 then
						if self.o("squeeze") then
							self.squeeze[dstGuid] = srcGuid
							local a = self.a("squeeze")
							local m = AVR:AddCircleWarning(AVRE.scene, srcName, true, 3, nil, a.r, a.g, a.b, a.a, nil, true)
							if m then
								m:SetTimer(8)
								self.circles[dstGuid] = m
								
								m = AVRMarkerMesh:New()
								m:SetFollowUnit(srcName)
								m:SetLine(true)
								m:SetColor(1.0, 0.2, 1.0, 0.8)
								AVRE.scene:AddMesh(m)
								self.markers[dstGuid] = m
							end
						end
					end
					
				elseif eventType == "UNIT_DIED" then
					
					-- Squeeze killed
					if self.squeeze[dstGuid] then
						if self.circles[dstGuid] then
							self.circles[dstGuid]:Remove()
							self.circles[dstGuid] = nil
						end
						if self.markers[dstGuid] then
							self.markers[dstGuid]:Remove()
							self.markers[dstGuid] = nil
						end
						self.squeeze[dstGuid] = nil
					end
					
					-- Squeezed player killed
					for k, v in pairs(self.squeeze) do
						if v == dstGuid then
							if self.circles[k] then
								self.circles[k]:Remove()
								self.circles[k] = nil
							end
							if self.markers[k] then
								self.markers[k]:Remove()
								self.markers[k] = nil
							end
							self.squeeze[k] = nil
						end
					end
				
				-- Malady of the Mind
				elseif eventType == "SPELL_AURA_APPLIED" then
					if (spellId == 63830 or spellId == 63881) and self.o("malady") then
						local a = self.a("malady")
						AVR:AddCircleWarning(AVRE.scene, dstName, true, 10, 4, a.r, a.g, a.b, a.a, nil, true)
					end
				
				-- Crusher Tentacle Spawned
				elseif eventType == "SPELL_CAST_SUCCESS" then
					if spellId == 64144 and srcName.a("crusher") then
						local a = srcName.a("crusher")
						if AVRE:GetCIDFromGUID(srcGuid) == 33966 then
							AVR:AddCircleWarning(AVRE.scene, srcName, true, 30, 5, a.r, a.g, a.b, a.a)
							local m = AVRMarkerMesh:New()
							m:SetFollowUnit(srcName.a("crusher"))
							m:SetLine(true)
							m:SetColor(1.0, 1.0, 1.0, 0.4)
							AVRE.scene:AddMesh(m)
							AVRE:ScheduleTimer(function() if m then m:Remove() end end, 30)
						end
					end
				end
			end,
		},



})
	
end
